-- Author: DanioMods
-- Date: 10.02.2020
-- Copyright (C): DanioMods, All Rights Reserved
-- Specialization for Bomet Musca
-- Version: Farming Simulator 19

-- !! PROHIBITION OF ANY EDITING, COPYING, DUPLICATION OR USE !!
-- !! VERBOT JEDER BEARBEITUNG, KOPIERUNG, DUPLIZIERUNG ODER VERWENDUNG !!
-- !! INTERDICTION DE TOUTE ÉDITION, COPIE, DUPLICATION OU UTILISATION !!
-- !! ЗАПРЕТ ЛЮБОГО РЕДАКТИРОВАНИЯ, КОПИРОВАНИЯ, ДУБЛИКАЦИИ ИЛИ ИСПОЛЬЗОВАНИЯ !!
-- !! DRAUDIMAS JOKIAM REDAGAVIMUI, KOPIJUOTI, DAUGINTI ARBA NAUDOTI !!
-- !! ZÁKAZ AKÉKOĽVEK ÚPRAVY, KOPÍROVANIA, DUPLIKÁCIE ALEBO POUŽITIA !!
-- !! DIVIETO DI QUALSIASI MODIFICA, COPIA, DUPLICAZIONE O UTILIZZO !!
-- !! ZAKAZ JAKICHKOLWIEK EDYCJI, KOPIOWANIA, POWIELANIA LUB UŻYWANIA !!

Musca = {}
Musca.MOD_NAME = g_currentModName

function Musca.prerequisitesPresent(specializations)
	return true
end;

function Musca.registerEvents(vehicleType)
	SpecializationUtil.registerEvent(vehicleType, "onWorkWidthChange")
end;

function Musca.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", Musca)
	SpecializationUtil.registerEventListener(vehicleType, "onUpdateTick", Musca)
	SpecializationUtil.registerEventListener(vehicleType, "onWorkWidthChange", Musca)
end;

function Musca.registerFunctions(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, "loadWorkWidthsFromXML", Musca.loadWorkWidthsFromXML)
	SpecializationUtil.registerFunction(vehicleType, "getActiveWorkWidth", Musca.getActiveWorkWidth)
	SpecializationUtil.registerFunction(vehicleType, "getIsWorkWidthActive", Musca.getIsWorkWidthActive)
end;

function Musca.registerOverwrittenFunctions(vehicleType)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "loadWorkAreaFromXML", Musca.loadWorkAreaFromXML)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getIsWorkAreaActive", Musca.getIsWorkAreaActive)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getAIMarkers", Musca.getAIMarkers)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getAISizeMarkers", Musca.getAISizeMarkers)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getAIImplementCollisionTriggers", Musca.getAIImplementCollisionTriggers)
end;

function Musca.initSpecialization()
	g_configurationManager:addConfigurationType("gaugeWheelsSmall", g_i18n:getText("configuration_gaugeWheels"), "gaugeWheelsSmall", nil, nil, nil, ConfigurationUtil.SELECTOR_MULTIOPTION)
	g_configurationManager:addConfigurationType("gaugeWheelsBig", g_i18n:getText("configuration_gaugeWheels"), "gaugeWheelsBig", nil, nil, nil, ConfigurationUtil.SELECTOR_MULTIOPTION)
	g_configurationManager:addConfigurationType("harrowSmall", g_i18n:getText("configuration_harrow"), "harrowSmall", nil, nil, nil, ConfigurationUtil.SELECTOR_MULTIOPTION)
	g_configurationManager:addConfigurationType("harrowBig", g_i18n:getText("configuration_harrow"), "harrowBig", nil, nil, nil, ConfigurationUtil.SELECTOR_MULTIOPTION)
	g_configurationManager:addConfigurationType("subsoilerModel", g_i18n:getText("configuration_subsoilerModel"), "subsoilerModel", nil, nil, nil, ConfigurationUtil.SELECTOR_MULTIOPTION)
end;

function Musca:onLoad(savegame)
	local specName = Musca.MOD_NAME .. ".musca"
	self.spec_musca = self["spec_" .. specName]
	local spec = self.spec_musca

	local gaugeWheelsSmallId = Utils.getNoNil(self.configurations.gaugeWheelsSmall, 1)
		if self.configurations["gaugeWheelsSmall"] ~= nil then
		ObjectChangeUtil.updateObjectChanges(self.xmlFile, "vehicle.gaugeWheelsSmall.gaugeWheelsSmallConfigurations.gaugeWheelsSmallConfiguration", gaugeWheelsSmallId, self.components, self)
	end;

	local gaugeWheelsBigId = Utils.getNoNil(self.configurations.gaugeWheelsBig, 1)
		if self.configurations["gaugeWheelsBig"] ~= nil then
		ObjectChangeUtil.updateObjectChanges(self.xmlFile, "vehicle.gaugeWheelsBig.gaugeWheelsBigConfigurations.gaugeWheelsBigConfiguration", gaugeWheelsBigId, self.components, self)
	end;

	local harrowSmallId = Utils.getNoNil(self.configurations.harrowSmall, 1)
		if self.configurations["harrowSmall"] ~= nil then
		ObjectChangeUtil.updateObjectChanges(self.xmlFile, "vehicle.harrowSmall.harrowSmallConfigurations.harrowSmallConfiguration", harrowSmallId, self.components, self)
	end;

	local harrowBigId = Utils.getNoNil(self.configurations.harrowBig, 1)
		if self.configurations["harrowBig"] ~= nil then
		ObjectChangeUtil.updateObjectChanges(self.xmlFile, "vehicle.harrowBig.harrowBigConfigurations.harrowBigConfiguration", harrowBigId, self.components, self)
	end;

	local subsoilerModelId = Utils.getNoNil(self.configurations.subsoilerModel, 1)
		if self.configurations["subsoilerModel"] ~= nil then
		ObjectChangeUtil.updateObjectChanges(self.xmlFile, "vehicle.subsoilerModel.subsoilerModelConfigurations.subsoilerModelConfiguration", subsoilerModelId, self.components, self)
	end;

	spec.lastActiveWorkWidth = nil
	spec.workWidths = {}
	i = 0

	while true do
		local key = string.format("vehicle.cultivator.workWidths.workWidth(%d)", i)

		if not hasXMLProperty(self.xmlFile, key) then
			break
		end;

		local workWidth = {}

		if self:loadWorkWidthsFromXML(self.xmlFile, key, workWidth) then
			table.insert(spec.workWidths, workWidth)

			workWidth.index = #spec.workWidths
		end;

		i = i + 1
	end;
end;

function Musca:loadWorkAreaFromXML(superFunc, workArea, xmlFile, key)
	local ret = superFunc(self, workArea, xmlFile, key)
	
	if workArea.type == WorkAreaType.DEFAULT then
		workArea.type = WorkAreaType.CULTIVATOR
	end;
	
	workArea.workWidth = getXMLInt(xmlFile, key .. "#workWidth")
	
	return ret
end;

function Musca:loadWorkWidthsFromXML(xmlFile, key, workWidth)
	workWidth.subsoilerModelConfigurationIndex = getXMLInt(self.xmlFile, key .. "#subsoilerModelConfigurationIndex")
	workWidth.ai = {
		leftMarker = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, key .. ".ai.areaMarkers#leftNode"), self.i3dMappings),
		rightMarker = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, key .. ".ai.areaMarkers#rightNode"), self.i3dMappings),
		backMarker = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, key .. ".ai.areaMarkers#backNode"), self.i3dMappings),
		sizeLeftMarker = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, key .. ".ai.sizeMarkers#leftNode"), self.i3dMappings),
		sizeRightMarker = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, key .. ".ai.sizeMarkers#rightNode"), self.i3dMappings),
		sizeBackMarker = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, key .. ".ai.sizeMarkers#backNode"), self.i3dMappings),
		collisionTrigger = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, key .. ".ai.collisionTrigger#node"), self.i3dMappings)
	}

	workWidth.objectChanges = {}

	ObjectChangeUtil.loadObjectChangeFromXML(self.xmlFile, key, workWidth.objectChanges, self.components, self)
	ObjectChangeUtil.setObjectChanges(workWidth.objectChanges, false)

	return true
end;

function Musca:onUpdateTick(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	local activeWorkWidth = self:getActiveWorkWidth()
	
	if activeWorkWidth ~= nil then
		local spec = self.spec_musca
		
		if activeWorkWidth ~= spec.lastActiveWorkWidth then
			SpecializationUtil.raiseEvent(self, "onWorkWidthChange", activeWorkWidth)
			
			spec.lastActiveWorkWidth = activeWorkWidth
		end;
	end;
end;

function Musca:getIsWorkAreaActive(superFunc, workArea)
	if workArea.workWidth ~= nil then
		local workWidth = self:getActiveWorkWidth()
		
		if workWidth ~= nil and workWidth.index ~= workArea.workWidth then
			return false
		end;
	end;
	
	return superFunc(self, workArea)
end;

function Musca:getActiveWorkWidth()
	local spec = self.spec_musca
	
	for _, workWidth in ipairs(spec.workWidths) do
		if self:getIsWorkWidthActive(workWidth) then
			return workWidth
		end;
	end;
	
	return nil
end;

function Musca:getIsWorkWidthActive(workWidth)
	local spec = self.spec_musca
	
	if workWidth.subsoilerModelConfigurationIndex ~= nil and (self.configurations.subsoilerModel or 1) ~= workWidth.subsoilerModelConfigurationIndex then
		return false
	end;
	
	return true
end;

function Musca:onWorkWidthChange(activeWorkWidth)
	local spec = self.spec_musca
	
	for _, workWidth in ipairs(spec.workWidths) do
		ObjectChangeUtil.setObjectChanges(workWidth.objectChanges, workWidth == activeWorkWidth)
	end;
end;

function Musca:getAIMarkers(superFunc)
	local spec = self.spec_aiImplement
	local workWidth = self:getActiveWorkWidth()
	
	if workWidth ~= nil and not spec.useAttributesOfAttachedImplement and workWidth.ai.rightMarker ~= nil then
		if spec.aiMarkersInverted then
			return workWidth.ai.rightMarker, workWidth.ai.leftMarker, workWidth.ai.backMarker, true
		else
			return workWidth.ai.rightMarker, workWidth.ai.leftMarker, workWidth.ai.backMarker, false
		end;
	end;
	
	return superFunc(self)
end;

function Musca:getAISizeMarkers(superFunc)
	local workWidth = self:getActiveWorkWidth()
	
	if workWidth ~= nil and workWidth.ai.sizeLeftMarker ~= nil then
		return workWidth.ai.sizeLeftMarker, workWidth.ai.sizeRightMarker, workWidth.ai.sizeBackMarker
	end;
	
	return superFunc(self)
end;

function Musca:getAIImplementCollisionTriggers(superFunc, collisionTriggers)
	local workWidth = self:getActiveWorkWidth()
	
	if workWidth ~= nil and workWidth.ai.collisionTrigger ~= nil then
		collisionTriggers[self] = workWidth.ai.collisionTrigger
	end;
	
	return superFunc(self, collisionTriggers)
end;